package xyz.tbvns.GeometryDash;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.dataformat.xml.XmlFactory;
import com.stanfy.gsonxml.GsonXml;
import com.stanfy.gsonxml.GsonXmlBuilder;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.FileUtils;
import org.xml.sax.SAXException;
import xyz.tbvns.XmlUtils;

import javax.imageio.ImageIO;
import javax.xml.parsers.ParserConfigurationException;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import static java.lang.Math.round;

@Slf4j
public class TextureExtractor {
    public static void extract(File plist, File textures, File output) throws IOException, ParserConfigurationException, SAXException {
        String config = XmlUtils.gamesheetToXML(removeLines(FileUtils.readFileToString(plist), 2));
        System.out.println(config);

        BufferedImage image = ImageIO.read(textures);
        String[] readableConfig = config
                .replace(" ", "")
                .split("\n");

        List<TextureReference> textureReferences = new ArrayList<>();
        TextureReference current = new TextureReference();
        for (String s : readableConfig) {
            if (s.equals("<dict>")) {
                if (current.isValid()) textureReferences.add(current);
                current = new TextureReference();
            } else if (s.startsWith("<name>")) {
                current.setName(
                        s.replace("<name>", "")
                         .replace("</name>", "")
                );
            } else if (s.startsWith("<textureRect>")) {
                String[] usableString =
                        s.replace("<textureRect>", "")
                         .replace("</textureRect>", "")
                         .replace("{", "")
                         .replace("}", "")
                         .split(",");
                current.setX(Float.parseFloat(usableString[0]));
                current.setY(Float.parseFloat(usableString[1]));
                current.setW(Float.parseFloat(usableString[2]));
                current.setH(Float.parseFloat(usableString[3]));
            } else if (s.startsWith("<textureRotated>")) {
                current.setRotated(
                        Boolean.parseBoolean(
                                s.replace("<textureRotated>", "")
                                 .replace("</textureRotated>", "")
                        )
                );
            }
        }

        for (TextureReference reference : textureReferences) {
            new Thread(() -> {
                BufferedImage crop;
                try {
                    if (!reference.isRotated()) {
                        crop = image.getSubimage(
                                round(reference.getX()),
                                round(reference.getY()),
                                round(reference.getW()),
                                round(reference.getH())
                        );
                    } else {
                        crop = rotate(
                                image.getSubimage(
                                        round(reference.getX()),
                                        round(reference.getY()),
                                        round(reference.getH()),
                                        round(reference.getW())
                                )
                        );
                    }
                    File file = new File(output.getPath() + "/" + reference.getName());
                    ImageIO.write(crop, "png", file);
                } catch (Exception e) {e.printStackTrace();}
            }).start();
        }
    }

    public static BufferedImage rotate(BufferedImage src) {
        int width = src.getWidth();
        int height = src.getHeight();
        BufferedImage output = new BufferedImage(height, width, src.getType());

        Graphics2D g2d = (Graphics2D)output.getGraphics();
        g2d.translate(height/2., width/2.);
        g2d.rotate((Math.PI/2) * 3);
        g2d.translate(-width/2., -height/2.);
        g2d.drawImage(src, 0,0,width, height,null);

        return output;
    }

    public static String removeLines(String s, int nb) {
        String[] data = s.split("\n");
        String newData = "";
        for (int i = 0; i < data.length; i++) {
            String st = data[i];
            if (i >= nb) {
                newData += st + "\n";
            }
        }
        return newData;
    }
}
